@@ -294,7 +294,15 @@ gtk_cell_view_style_set (GtkWidget *widget,
 static void
 gtk_cell_view_finalize (GObject *object)
 {
-  gtk_cell_view_cell_layout_clear (GTK_CELL_LAYOUT (object));
+  GtkCellView *cellview = GTK_CELL_VIEW (object);
+
+  gtk_cell_view_cell_layout_clear (GTK_CELL_LAYOUT (cellview));
+
+  if (cellview->priv->model)
+     g_object_unref (cellview->priv->model);
+
+  if (cellview->priv->displayed_row)
+     gtk_tree_row_reference_free (cellview->priv->displayed_row);
 
   if (G_OBJECT_CLASS (parent_class)->finalize)
     (* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -696,17 +704,18 @@ gtk_cell_view_cell_layout_clear_attributes (GtkCellLayout   *layout,
   g_return_if_fail (GTK_IS_CELL_RENDERER (renderer));
 
   info = gtk_cell_view_get_cell_info (cellview, renderer);
-  g_return_if_fail (info != NULL);
-
-  list = info->attributes;
-  while (list && list->next)
+  if (info != NULL)
     {
-      g_free (list->data);
-      list = list->next->next;
+      list = info->attributes;
+      while (list && list->next)
+	{
+	  g_free (list->data);
+	  list = list->next->next;
+	}
+      
+      g_slist_free (info->attributes);
+      info->attributes = NULL;
     }
-
-  g_slist_free (info->attributes);
-  info->attributes = NULL;
 }
 
 static void
