@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <string.h>
 #include <gobject/gvaluecollector.h>
 #include "gtktreemodel.h"
@@ -1638,7 +1639,9 @@ gtk_list_store_reorder_func (gconstpointer a,
 /**
  * gtk_list_store_reorder:
  * @store: A #GtkListStore.
- * @new_order: An integer array indicating the new order for the list.
+ * @new_order: an array of integers mapping the new position of each child
+ *      to its old position before the re-ordering,
+ *      i.e. @new_order<literal>[newpos] = oldpos</literal>.
  *
  * Reorders @store to follow the order indicated by @new_order. Note that
  * this function only works with unsorted stores.
@@ -1664,7 +1667,7 @@ gtk_list_store_reorder (GtkListStore *store,
 
   for (i = 0; i < store->length; i++)
     {
-      sort_array[i].offset = new_order[i];
+      sort_array[new_order[i]].offset = i;
       sort_array[i].el = current_list;
 
       current_list = current_list->next;
@@ -2092,7 +2095,8 @@ gtk_list_store_sort (GtkListStore *list_store)
   if (list_store->length <= 1)
     return;
 
-  g_assert (GTK_LIST_STORE_IS_SORTED (list_store));
+  if (!GTK_LIST_STORE_IS_SORTED (list_store))
+    return;
 
   list = G_SLIST (list_store->root);
 
@@ -2350,19 +2354,22 @@ gtk_list_store_set_sort_column_id (GtkTreeSortable  *sortable,
       (list_store->order == order))
     return;
 
-  if (sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
+  if (sort_column_id != -2)
     {
-      GtkTreeDataSortHeader *header = NULL;
-
-      header = _gtk_tree_data_list_get_header (list_store->sort_list, sort_column_id);
-
-      /* We want to make sure that we have a function */
-      g_return_if_fail (header != NULL);
-      g_return_if_fail (header->func != NULL);
-    }
-  else
-    {
-      g_return_if_fail (list_store->default_sort_func != NULL);
+      if (sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
+	{
+	  GtkTreeDataSortHeader *header = NULL;
+	  
+	  header = _gtk_tree_data_list_get_header (list_store->sort_list, sort_column_id);
+	  
+	  /* We want to make sure that we have a function */
+	  g_return_if_fail (header != NULL);
+	  g_return_if_fail (header->func != NULL);
+	}
+      else
+	{
+	  g_return_if_fail (list_store->default_sort_func != NULL);
+	}
     }
 
 
